/*
 * STAR screenblanker v0.2
 *
 *  (c)1996 Nobumasa Tamaoki
 *  E-mail: uranus@hipecs.hokudai.ac.jp
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include "vga.h"
#include "star.h"

typedef struct{
  int x, y, z;
} POINT;

POINT stars_3D[STAR_NUM];
POINT stars_2D[STAR_NUM];

static inline void perspective(POINT *p, POINT *q)
{
  int w;
  
  w = (EYE_Z << FP) / (EYE_Z + p->z);
  
  if(p->y)
    q->y = ((p->y * w) >> FP) + (Y_MAX / 2);
  else
    q->y = (Y_MAX / 2);
  
  if(p->x)
    q->x = ((p->x * w) >> FP) + (X_MAX / 2);
  else
    q->x = (X_MAX / 2);
}

void handler(int signal)
{
  /* restore screen mode */
  vga_setmode(TEXT);
  
  exit(-1);
}

static void make_newstar(int n)
{
  stars_3D[n].x = (rand() % X_RANGE) - (X_RANGE / 2);
  stars_3D[n].y = (rand() % Y_RANGE) - (Y_RANGE / 2);
  stars_3D[n].z = STAR_Z;
  
  stars_2D[n].x = 0;
  stars_2D[n].y = 0;
}

static int initialize(void)
{
  int i;
  
  /* initialize svgalib */
  if(vga_init()){
    fprintf(stderr, "error: vga_init() failed.\n");
    return 1;
  }
  
  /* set graphic mode */
  if(vga_setmode(GRAPHIC_MODE)){
    fprintf(stderr, "error: invalid graphic mode (%d)\n", GRAPHIC_MODE);
    return 2;
  }
  
  /* set SIGQUIT handler */
  signal(SIGQUIT, handler);
  
  /* initiaize rand() */
  srand(getpid());
  
  /* initialize stars */
  for(i = 0; i < STAR_NUM; i++)
    make_newstar(i);
  
  /* initialize palette */
  for(i = 0; i < 256; i++)
    vga_setpalette(i, i / 4, i / 4, i / 4);
  
  return 0;
}

int main(void)
{
  int i;
  POINT *p;
  
  if(initialize())
    exit(-1);
  
  /* main loop */
  while(1){
    /* wait */
    usleep(USLEEP_TIME);
    
    /* display loop */
    for(i = 0; i < STAR_NUM; i++){
      /* clear */
      vga_setcolor(0);
      vga_drawpixel(stars_2D[i].x, stars_2D[i].y);
      
      /* move */
      stars_3D[i].z -= STAR_SPEED;
      
      /* clipping */
      /* z */
      if(stars_3D[i].z > -EYE_Z){
	perspective(&stars_3D[i], &stars_2D[i]);
	p = &stars_2D[i];
	/* x, y */
	if(p->x < 0 || p->x >= X_MAX || p->y < 0 || p->y >= Y_MAX){
	  /* generate new star */
	  make_newstar(i);
	}
	else{
	  /* display */
	  vga_setcolor(((stars_3D[i].z > 0) ? (255 - stars_3D[i].z) : 255));
	  vga_drawpixel(p->x, p->y);
	}
      }
      else{
	/* generate new star */
	make_newstar(i);
      }
    }
  }
  
  return 0;
}
